@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014-2017 Christian Schoenebeck
+    Copyright (c) 2014-2019 Christian Schoenebeck
     
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -941,7 +941,7 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     }
 
     m_buttonBox.set_layout(Gtk::BUTTONBOX_END);
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
     m_buttonBox.set_margin(5);
 #else
     m_buttonBox.set_border_width(5);
@@ -966,6 +966,11 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     show_all_children();
 #endif
 
+    Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
+        sigc::mem_fun(*this, &CombineInstrumentsDialog::on_show_tooltips_changed)
+    );
+    on_show_tooltips_changed();
+
     // show a warning to user if he uses a .gig in v2 format
     if (gig->pVersion->major < 3) {
         Glib::ustring txt = _(
@@ -1023,10 +1028,6 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
     const Gtk::SelectionData& selection_data, guint, guint time)
 {
     printf("Drag data received\n");
-    if (&selection_data == NULL) {
-        printf("!selection_data\n");
-        return;
-    }
     if (!selection_data.get_data()) {
         printf("selection_data.get_data() == NULL\n");
         return;
@@ -1037,7 +1038,7 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
         printf("!src\n");
         return;
     }
-    printf("src=%d\n", src);
+    printf("src=%ld\n", (size_t)src);
 
     gig::Instrument* dst = NULL;
     {
@@ -1249,6 +1250,15 @@ void CombineInstrumentsDialog::onSelectionChanged() {
     }
 }
 
+void CombineInstrumentsDialog::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    m_treeView.set_has_tooltip(b);
+    m_iconView.set_has_tooltip(b);
+
+    set_has_tooltip(b);
+}
+
 bool CombineInstrumentsDialog::fileWasChanged() const {
     return m_fileWasChanged;
 }
