@@ -1,5 +1,5 @@
 /*
-    Copyright (c) MMXVII Christian Schoenebeck
+    Copyright (c) MMXVII - MMXIX Christian Schoenebeck
 
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -232,7 +232,7 @@ void MacroEditor::buildTreeView(const Gtk::TreeModel::Row& parentRow, const Seri
         const Serialization::Member& member = parentObject.members()[iMember];
         const Serialization::Object& object = m_macro.objectByUID(member.uid());
         
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
         //HACK: on GTKMM 3.9x append() below requires TreeNodeChildren, parentRow.children() returns TreeNodeConstChildren though, probably going to be fixed before final GTKMM4 release though.
         const Gtk::TreeNodeConstChildren& children = parentRow.children();
         Gtk::TreeNodeChildren* const pChildren = (Gtk::TreeNodeChildren* const) &children;
@@ -504,7 +504,7 @@ bool MacroEditor::onWindowDeleteP(GdkEventAny* /*e*/) {
 
     //gchar* msg = g_strdup_printf(_("Apply changes to macro \"%s\" before closing?"),
     //                             m_macroOriginal->Name.c_str());
-    gchar* msg = g_strdup_printf(_("Apply changes to macro before closing?"));
+    gchar* msg = g_strdup(_("Apply changes to macro before closing?"));
     Gtk::MessageDialog dialog(*this, msg, false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE);
     g_free(msg);
     dialog.set_secondary_text(_("If you close without applying, your changes will be lost."));
