@@ -1,5 +1,5 @@
 /*                                                         -*- c++ -*-
- * Copyright (C) 2011-2017 Andreas Persson
+ * Copyright (C) 2011-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 
 #ifdef CAIROMM_HEADER_FILE
 # include CAIROMM_HEADER_FILE(cairommconfig.h)
-#else
+#elif HAVE_CAIROMMCONFIG_H
 # include <cairommconfig.h>
 #endif
 
@@ -50,8 +50,9 @@
 # include <gtkmmconfig.h>
 #endif
 
-#ifndef HAS_PANGOMM_CPP11_ENUMS // pangomm > 2.40 : <- just a "guess"
-# if PANGOMM_MAJOR_VERSION > 2 || (PANGOMM_MAJOR_VERSION == 2 && PANGOMM_MINOR_VERSION > 40)
+#ifndef HAS_PANGOMM_CPP11_ENUMS
+   // new enums introduced in unstable pangomm 2.41.3, but not in stable 2.42
+# if PANGOMM_MAJOR_VERSION > 2 || (PANGOMM_MAJOR_VERSION == 2 && ((PANGOMM_MINOR_VERSION == 41 && PANGOMM_MICRO_VERSION >= 3) || PANGOMM_MINOR_VERSION > 42))
 #  define HAS_PANGOMM_CPP11_ENUMS 1
 # else
 #  define HAS_PANGOMM_CPP11_ENUMS 0
@@ -61,7 +62,7 @@
 // Gtk::UIManager had been replaced by Gtk::Builder in GTKMM 3 and removed in GTKMM 4
 // Gtk::ActionGroup had been replaced by Gio::ActionGroup in GTKMM 3 and removed in GTKMM 4
 #if !defined(USE_GTKMM_BUILDER) || !defined(USE_GLIB_ACTION) // gtkmm > 3.22 :
-# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 #  define USE_GTKMM_BUILDER 1
 #  define USE_GLIB_ACTION 1
 # else
@@ -79,7 +80,7 @@
 #endif
 
 #ifndef HAS_GTKMM_CPP11_ENUMS // gtkmm > 3.22
-# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 #  define HAS_GTKMM_CPP11_ENUMS 1
 # else
 #  define HAS_GTKMM_CPP11_ENUMS 0
@@ -88,7 +89,7 @@
 
 // alignment.h had been removed in GTKMM 4
 #ifndef HAS_GTKMM_ALIGNMENT // gtkmm > 3.22 :
-# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 #  define HAS_GTKMM_ALIGNMENT 0
 # else
 #  define HAS_GTKMM_ALIGNMENT 1
@@ -99,7 +100,7 @@
 // stock.h had been removed in GTKMM 4
 // Gtk::VBox, Gtk::HBox and Gtk::HButtonBox had been replaced by Gtk::Box in GTKMM 3 and removed in GTKMM 4
 #if !defined(USE_GTKMM_GRID) || !defined(HAS_GTKMM_STOCK) || !defined(USE_GTKMM_BOX) || !defined(USE_GTKMM_PANED) // gtkmm > 3.22 :
-# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+# if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 #  define USE_GTKMM_GRID 1
 #  define HAS_GTKMM_STOCK 0
 #  define USE_GTKMM_BOX 1
@@ -422,4 +423,14 @@ namespace Glib {
 
 #endif // glibmm 2.31.2
 
+
+// glibmm 2.47.3
+
+#if (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION == 47 && GLIBMM_MICRO_VERSION < 3) || \
+    (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION < 47) || GLIBMM_MAJOR_VERSION < 2
+
+#define GLIB_THREADS
+
+#endif
+
 #endif // GIGEDIT_COMPAT_H
