@@ -18,13 +18,6 @@
  */
 
 #include "compat.h"
-// threads.h must be included first to be able to build with
-// G_DISABLE_DEPRECATED
-#if (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION == 31 && GLIBMM_MICRO_VERSION >= 2) || \
-    (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION > 31) || GLIBMM_MAJOR_VERSION > 2
-#include <glibmm/threads.h>
-#endif
-
 #include "dimensionmanager.h"
 
 #if HAS_GTKMM_STOCK
@@ -278,7 +271,7 @@ DimensionManager::DimensionManager() :
     scrolledWindow.show();
     vbox.pack_start(buttonBox, Gtk::PACK_SHRINK);
     buttonBox.set_layout(Gtk::BUTTONBOX_END);
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
     buttonBox.set_margin(5);
 #else
     buttonBox.set_border_width(5);
@@ -325,6 +318,20 @@ DimensionManager::DimensionManager() :
 #if HAS_GTKMM_SHOW_ALL_CHILDREN
     show_all_children();
 #endif
+
+    Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
+        sigc::mem_fun(*this, &DimensionManager::on_show_tooltips_changed)
+    );
+    on_show_tooltips_changed();
+}
+
+void DimensionManager::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    treeView.set_has_tooltip(b);
+    allRegionsCheckBox.set_has_tooltip(b);
+
+    set_has_tooltip(b);
 }
 
 bool DimensionManager::allRegions() const {
@@ -353,7 +360,7 @@ struct _DimDef {
 };
 typedef std::map<gig::dimension_t, _DimDef> _Dimensions;
 
-// update all GUI elements according to current gig::Region informations
+// update all GUI elements according to current gig::Region information
 void DimensionManager::refreshManager() {
     set_sensitive(false);
     refTableModel->clear();
