@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2017 Andreas Persson
+ * Copyright (C) 2006-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,7 @@
 
 #include "global.h"
 #include "compat.h"
+#include "Settings.h"
 #include <gtkmm/box.h>
 #include "dimregionchooser.h"
 #include <cairomm/context.h>
@@ -288,7 +289,7 @@ DimRegionChooser::DimRegionChooser(Gtk::Window& window) :
 #endif // USE_GTKMM_BUILDER
 
 
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 # warning GTKMM4 event registration code missing for dimregionchooser!
     //add_events(Gdk::EventMask::BUTTON_PRESS_MASK);
 #else
@@ -302,6 +303,11 @@ DimRegionChooser::DimRegionChooser(Gtk::Window& window) :
         "Right click here for options on altering dimension zones. Press and "
         "hold CTRL key for selecting multiple dimension zones simultaniously."
     ));
+
+    Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
+        sigc::mem_fun(*this, &DimRegionChooser::on_show_tooltips_changed)
+    );
+    on_show_tooltips_changed();
     
     window.signal_key_press_event().connect(
         sigc::mem_fun(*this, &DimRegionChooser::onKeyPressed)
@@ -315,6 +321,12 @@ DimRegionChooser::~DimRegionChooser()
 {
 }
 
+void DimRegionChooser::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    set_has_tooltip(b);
+}
+
 void DimRegionChooser::setModifyBothChannels(bool b) {
     modifybothchannels = b;
     // redraw required parts
@@ -1010,7 +1022,7 @@ bool DimRegionChooser::on_button_release_event(GdkEventButton* event)
 # if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
         Glib::wrap(event->device, true)->ungrab(event->time);
 # else
-        gdk_device_ungrab(Glib::wrap(event->device, true)->gobj(), event->time);
+        Glib::wrap(event->device, true)->get_seat()->ungrab();
 # endif
 #endif
         resize.active = false;
@@ -1050,18 +1062,15 @@ bool DimRegionChooser::on_button_press_event(GdkEventButton* event)
                                                   Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW),
                                                   event->time);
 # else
-            gdk_device_grab(
-                Glib::wrap(event->device, true)->gobj(),
-                get_window()->gobj(),
-                GDK_OWNERSHIP_NONE,
+            Glib::wrap(event->device, true)->get_seat()->grab(
+                get_window(),
+                Gdk::SeatCapabilities::SEAT_CAPABILITY_ALL_POINTING,
                 false,
-                GdkEventMask(GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
-                             GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK),
                 Gdk::Cursor::create(
                     Glib::wrap(event->device, true)->get_seat()->get_display(),
                     Gdk::SB_H_DOUBLE_ARROW
-                )->gobj(),
-                event->time
+                ),
+                reinterpret_cast<GdkEvent*>(event)
             );
 # endif
 #endif
