@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2017 Andreas Persson
+ * Copyright (C) 2006-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,8 @@
 # include <gtkmm/table.h>
 #endif
 
+#include "Settings.h"
+
 VelocityCurve::VelocityCurve(double (gig::DimensionRegion::*getter)(uint8_t)) :
     getter(getter), dimreg(0) {
     set_size_request(80, 80);
@@ -187,6 +189,16 @@ EGStateOptions::EGStateOptions() : HBox(),
     ));
 }
 
+void EGStateOptions::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    checkBoxAttack.set_has_tooltip(b);
+    checkBoxAttackHold.set_has_tooltip(b);
+    checkBoxDecay1.set_has_tooltip(b);
+    checkBoxDecay2.set_has_tooltip(b);
+    checkBoxRelease.set_has_tooltip(b);
+}
+
 
 DimRegionEdit::DimRegionEdit() :
     velocity_curve(&gig::DimensionRegion::GetVelocityAttenuation),
@@ -260,6 +272,8 @@ DimRegionEdit::DimRegionEdit() :
     eCrossfade_out_start(_("Crossfade-out start")),
     eCrossfade_out_end(_("Crossfade-out end")),
     ePitchTrack(_("Pitch track")),
+    eSustainReleaseTrigger(_("Sustain Release Trigger")),
+    eNoNoteOffReleaseTrigger(_("No note-off release trigger")),
     eDimensionBypass(_("Dimension bypass")),
     ePan(_("Pan"), -64, 63),
     eSelfMask(_("Kill lower velocity voices (a.k.a \"Self mask\")")),
@@ -279,7 +293,7 @@ DimRegionEdit::DimRegionEdit() :
     eGain(_("Gain"), -96, 0, 2, -655360),
     eGainPlus6(_("Gain +6dB"), eGain, 6 * -655360),
     eSampleLoopEnabled(_("Enabled")),
-    eSampleLoopStart(_("Loop start positon")),
+    eSampleLoopStart(_("Loop start position")),
     eSampleLoopLength(_("Loop size")),
     eSampleLoopType(_("Loop type")),
     eSampleLoopInfinite(_("Infinite loop")),
@@ -307,26 +321,16 @@ DimRegionEdit::DimRegionEdit() :
             &gig::DimensionRegion::EG1ControllerDecayInfluence);
     connect(eEG1ControllerReleaseInfluence,
             &gig::DimensionRegion::EG1ControllerReleaseInfluence);
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG1Options.AttackCancel));
-        connect(eEG1StateOptions.checkBoxAttack, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG1Options.AttackHoldCancel));
-        connect(eEG1StateOptions.checkBoxAttackHold, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG1Options.Decay1Cancel));
-        connect(eEG1StateOptions.checkBoxDecay1, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG1Options.Decay2Cancel));
-        connect(eEG1StateOptions.checkBoxDecay2, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG1Options.ReleaseCancel));
-        connect(eEG1StateOptions.checkBoxRelease, mp.pmember);
-    }
+    connect(eEG1StateOptions.checkBoxAttack, &gig::DimensionRegion::EG1Options,
+            &gig::eg_opt_t::AttackCancel);
+    connect(eEG1StateOptions.checkBoxAttackHold, &gig::DimensionRegion::EG1Options,
+            &gig::eg_opt_t::AttackHoldCancel);
+    connect(eEG1StateOptions.checkBoxDecay1, &gig::DimensionRegion::EG1Options,
+            &gig::eg_opt_t::Decay1Cancel);
+    connect(eEG1StateOptions.checkBoxDecay2, &gig::DimensionRegion::EG1Options,
+            &gig::eg_opt_t::Decay2Cancel);
+    connect(eEG1StateOptions.checkBoxRelease, &gig::DimensionRegion::EG1Options,
+            &gig::eg_opt_t::ReleaseCancel);
     connect(eLFO1Frequency, &gig::DimensionRegion::LFO1Frequency);
     connect(eLFO1InternalDepth, &gig::DimensionRegion::LFO1InternalDepth);
     connect(eLFO1ControlDepth, &gig::DimensionRegion::LFO1ControlDepth);
@@ -348,26 +352,16 @@ DimRegionEdit::DimRegionEdit() :
             &gig::DimensionRegion::EG2ControllerDecayInfluence);
     connect(eEG2ControllerReleaseInfluence,
             &gig::DimensionRegion::EG2ControllerReleaseInfluence);
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG2Options.AttackCancel));
-        connect(eEG2StateOptions.checkBoxAttack, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG2Options.AttackHoldCancel));
-        connect(eEG2StateOptions.checkBoxAttackHold, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG2Options.Decay1Cancel));
-        connect(eEG2StateOptions.checkBoxDecay1, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG2Options.Decay2Cancel));
-        connect(eEG2StateOptions.checkBoxDecay2, mp.pmember);
-    }
-    {
-        ClassMemberPtr<gig::DimensionRegion, bool> mp(offsetof(gig::DimensionRegion, EG2Options.ReleaseCancel));
-        connect(eEG2StateOptions.checkBoxRelease, mp.pmember);
-    }
+    connect(eEG2StateOptions.checkBoxAttack, &gig::DimensionRegion::EG2Options,
+            &gig::eg_opt_t::AttackCancel);
+    connect(eEG2StateOptions.checkBoxAttackHold, &gig::DimensionRegion::EG2Options,
+            &gig::eg_opt_t::AttackHoldCancel);
+    connect(eEG2StateOptions.checkBoxDecay1, &gig::DimensionRegion::EG2Options,
+            &gig::eg_opt_t::Decay1Cancel);
+    connect(eEG2StateOptions.checkBoxDecay2, &gig::DimensionRegion::EG2Options,
+            &gig::eg_opt_t::Decay2Cancel);
+    connect(eEG2StateOptions.checkBoxRelease, &gig::DimensionRegion::EG2Options,
+            &gig::eg_opt_t::ReleaseCancel);
     connect(eLFO2Frequency, &gig::DimensionRegion::LFO2Frequency);
     connect(eLFO2InternalDepth, &gig::DimensionRegion::LFO2InternalDepth);
     connect(eLFO2ControlDepth, &gig::DimensionRegion::LFO2ControlDepth);
@@ -415,6 +409,8 @@ DimRegionEdit::DimRegionEdit() :
     connect(eCrossfade_out_start, &DimRegionEdit::set_Crossfade_out_start);
     connect(eCrossfade_out_end, &DimRegionEdit::set_Crossfade_out_end);
     connect(ePitchTrack, &gig::DimensionRegion::PitchTrack);
+    connect(eSustainReleaseTrigger, &gig::DimensionRegion::SustainReleaseTrigger);
+    connect(eNoNoteOffReleaseTrigger, &gig::DimensionRegion::NoNoteOffReleaseTrigger);
     connect(eDimensionBypass, &gig::DimensionRegion::DimensionBypass);
     connect(ePan, &gig::DimensionRegion::Pan);
     connect(eSelfMask, &gig::DimensionRegion::SelfMask);
@@ -879,6 +875,21 @@ DimRegionEdit::DimRegionEdit() :
     rowno++;
 
     addProp(eReleaseTriggerDecay);
+    {
+        const char* choices[] = { _("off"), _("on (max. velocity)"), _("on (key velocity)"), 0 };
+        static const gig::sust_rel_trg_t values[] = {
+            gig::sust_rel_trg_none,
+            gig::sust_rel_trg_maxvelocity,
+            gig::sust_rel_trg_keyvelocity
+        };
+        eSustainReleaseTrigger.set_choices(choices, values);
+    }
+    eSustainReleaseTrigger.set_tip(_(
+        "By default release trigger samples are played on note-off events only. "
+        "This option allows to play release trigger sample on sustain pedal up "
+        "events as well. NOTE: This is a format extension!"
+    ));
+    addProp(eSustainReleaseTrigger);
     {
         const char* choices[] = { _("none"), _("effect4depth"), _("effect5depth"), 0 };
         static const gig::dim_bypass_ctrl_t values[] = {
@@ -888,6 +899,12 @@ DimRegionEdit::DimRegionEdit() :
         };
         eDimensionBypass.set_choices(choices, values);
     }
+    eNoNoteOffReleaseTrigger.set_tip(_(
+        "By default release trigger samples are played on note-off events only. "
+        "If this option is checked, then no release trigger sample is played "
+        "when releasing a note. NOTE: This is a format extension!"
+    ));
+    addProp(eNoNoteOffReleaseTrigger);
     addProp(eDimensionBypass);
     eSelfMask.widget.set_tooltip_text(_(
         "If enabled: new notes with higher velocity value will stop older "
@@ -960,6 +977,12 @@ DimRegionEdit::DimRegionEdit() :
     append_page(*table[4], _("Filter (2)"));
     append_page(*table[5], _("Pitch"));
     append_page(*table[6], _("Misc"));
+
+    Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
+        sigc::mem_fun(*this, &DimRegionEdit::on_show_tooltips_changed)
+    );
+
+    on_show_tooltips_changed();
 }
 
 DimRegionEdit::~DimRegionEdit()
@@ -1067,6 +1090,19 @@ Gtk::Label* DimRegionEdit::addHeader(const char* text)
     return label;
 }
 
+void DimRegionEdit::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    buttonSelectSample.set_has_tooltip(b);
+    buttonNullSampleReference->set_has_tooltip(b);
+    wSample->set_has_tooltip(b);
+
+    eEG1StateOptions.on_show_tooltips_changed();
+    eEG2StateOptions.on_show_tooltips_changed();
+
+    set_has_tooltip(b);
+}
+
 void DimRegionEdit::nextPage()
 {
     if (firstRowInBlock < rowno - 1)
@@ -1232,6 +1268,8 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     eCrossfade_out_start.set_value(d->Crossfade.out_start);
     eCrossfade_out_end.set_value(d->Crossfade.out_end);
     ePitchTrack.set_value(d->PitchTrack);
+    eSustainReleaseTrigger.set_value(d->SustainReleaseTrigger);
+    eNoNoteOffReleaseTrigger.set_value(d->NoNoteOffReleaseTrigger);
     eDimensionBypass.set_value(d->DimensionBypass);
     ePan.set_value(d->Pan);
     eSelfMask.set_value(d->SelfMask);
@@ -1637,116 +1675,116 @@ sigc::signal<void, gig::Sample*/*old*/, gig::Sample*/*new*/>& DimRegionEdit::sig
 }
 
 
-void DimRegionEdit::set_UnityNote(gig::DimensionRegion* d, uint8_t value)
+void DimRegionEdit::set_UnityNote(gig::DimensionRegion& d, uint8_t value)
 {
-    d->UnityNote = value;
+    d.UnityNote = value;
 }
 
-void DimRegionEdit::set_FineTune(gig::DimensionRegion* d, int16_t value)
+void DimRegionEdit::set_FineTune(gig::DimensionRegion& d, int16_t value)
 {
-    d->FineTune = value;
+    d.FineTune = value;
 }
 
-void DimRegionEdit::set_Crossfade_in_start(gig::DimensionRegion* d,
+void DimRegionEdit::set_Crossfade_in_start(gig::DimensionRegion& d,
                                            uint8_t value)
 {
-    d->Crossfade.in_start = value;
-    if (d->Crossfade.in_end < value) set_Crossfade_in_end(d, value);
+    d.Crossfade.in_start = value;
+    if (d.Crossfade.in_end < value) set_Crossfade_in_end(d, value);
 }
 
-void DimRegionEdit::set_Crossfade_in_end(gig::DimensionRegion* d,
+void DimRegionEdit::set_Crossfade_in_end(gig::DimensionRegion& d,
                                          uint8_t value)
 {
-    d->Crossfade.in_end = value;
-    if (value < d->Crossfade.in_start) set_Crossfade_in_start(d, value);
-    if (value > d->Crossfade.out_start) set_Crossfade_out_start(d, value);
+    d.Crossfade.in_end = value;
+    if (value < d.Crossfade.in_start) set_Crossfade_in_start(d, value);
+    if (value > d.Crossfade.out_start) set_Crossfade_out_start(d, value);
 }
 
-void DimRegionEdit::set_Crossfade_out_start(gig::DimensionRegion* d,
+void DimRegionEdit::set_Crossfade_out_start(gig::DimensionRegion& d,
                                             uint8_t value)
 {
-    d->Crossfade.out_start = value;
-    if (value < d->Crossfade.in_end) set_Crossfade_in_end(d, value);
-    if (value > d->Crossfade.out_end) set_Crossfade_out_end(d, value);
+    d.Crossfade.out_start = value;
+    if (value < d.Crossfade.in_end) set_Crossfade_in_end(d, value);
+    if (value > d.Crossfade.out_end) set_Crossfade_out_end(d, value);
 }
 
-void DimRegionEdit::set_Crossfade_out_end(gig::DimensionRegion* d,
+void DimRegionEdit::set_Crossfade_out_end(gig::DimensionRegion& d,
                                           uint8_t value)
 {
-    d->Crossfade.out_end = value;
-    if (value < d->Crossfade.out_start) set_Crossfade_out_start(d, value);
+    d.Crossfade.out_end = value;
+    if (value < d.Crossfade.out_start) set_Crossfade_out_start(d, value);
 }
 
-void DimRegionEdit::set_Gain(gig::DimensionRegion* d, int32_t value)
+void DimRegionEdit::set_Gain(gig::DimensionRegion& d, int32_t value)
 {
-    d->SetGain(value);
+    d.SetGain(value);
 }
 
-void DimRegionEdit::set_LoopEnabled(gig::DimensionRegion* d, bool value)
+void DimRegionEdit::set_LoopEnabled(gig::DimensionRegion& d, bool value)
 {
     if (value) {
         // create a new sample loop in case there is none yet
-        if (!d->SampleLoops) {
-            DimRegionChangeGuard(this, d);
+        if (!d.SampleLoops) {
+            DimRegionChangeGuard(this, &d);
 
             DLS::sample_loop_t loop;
             loop.LoopType = gig::loop_type_normal;
             // loop the whole sample by default
             loop.LoopStart  = 0;
             loop.LoopLength =
-                (d->pSample) ? d->pSample->SamplesTotal : 0;
-            d->AddSampleLoop(&loop);
+                (d.pSample) ? d.pSample->SamplesTotal : 0;
+            d.AddSampleLoop(&loop);
         }
     } else {
-        if (d->SampleLoops) {
-            DimRegionChangeGuard(this, d);
+        if (d.SampleLoops) {
+            DimRegionChangeGuard(this, &d);
 
             // delete ALL existing sample loops
-            while (d->SampleLoops) {
-                d->DeleteSampleLoop(&d->pSampleLoops[0]);
+            while (d.SampleLoops) {
+                d.DeleteSampleLoop(&d.pSampleLoops[0]);
             }
         }
     }
 }
 
-void DimRegionEdit::set_LoopType(gig::DimensionRegion* d, uint32_t value)
+void DimRegionEdit::set_LoopType(gig::DimensionRegion& d, uint32_t value)
 {
-    if (d->SampleLoops) d->pSampleLoops[0].LoopType = value;
+    if (d.SampleLoops) d.pSampleLoops[0].LoopType = value;
 }
 
-void DimRegionEdit::set_LoopStart(gig::DimensionRegion* d, uint32_t value)
+void DimRegionEdit::set_LoopStart(gig::DimensionRegion& d, uint32_t value)
 {
-    if (d->SampleLoops) {
-        d->pSampleLoops[0].LoopStart =
-            d->pSample ?
-            std::min(value, uint32_t(d->pSample->SamplesTotal -
-                                     d->pSampleLoops[0].LoopLength)) :
+    if (d.SampleLoops) {
+        d.pSampleLoops[0].LoopStart =
+            d.pSample ?
+            std::min(value, uint32_t(d.pSample->SamplesTotal -
+                                     d.pSampleLoops[0].LoopLength)) :
             0;
     }
 }
 
-void DimRegionEdit::set_LoopLength(gig::DimensionRegion* d, uint32_t value)
+void DimRegionEdit::set_LoopLength(gig::DimensionRegion& d, uint32_t value)
 {
-    if (d->SampleLoops) {
-        d->pSampleLoops[0].LoopLength =
-            d->pSample ?
-            std::min(value, uint32_t(d->pSample->SamplesTotal -
-                                     d->pSampleLoops[0].LoopStart)) :
+    if (d.SampleLoops) {
+        d.pSampleLoops[0].LoopLength =
+            d.pSample ?
+            std::min(value, uint32_t(d.pSample->SamplesTotal -
+                                     d.pSampleLoops[0].LoopStart)) :
             0;
     }
 }
 
-void DimRegionEdit::set_LoopInfinite(gig::DimensionRegion* d, bool value)
+void DimRegionEdit::set_LoopInfinite(gig::DimensionRegion& d, bool value)
 {
-    if (d->pSample) {
-        if (value) d->pSample->LoopPlayCount = 0;
-        else if (d->pSample->LoopPlayCount == 0) d->pSample->LoopPlayCount = 1;
+    if (d.pSample) {
+        if (value) d.pSample->LoopPlayCount = 0;
+        else if (d.pSample->LoopPlayCount == 0) d.pSample->LoopPlayCount = 1;
     }
 }
 
-void DimRegionEdit::set_LoopPlayCount(gig::DimensionRegion* d, uint32_t value)
+void DimRegionEdit::set_LoopPlayCount(gig::DimensionRegion& d, uint32_t value)
 {
-    if (d->pSample) d->pSample->LoopPlayCount = value;
+    if (d.pSample) d.pSample->LoopPlayCount = value;
 }
 
 void DimRegionEdit::nullOutSampleReference() {
