@@ -1,5 +1,5 @@
 /*                                                         -*- c++ -*-
- * Copyright (C) 2006-2017 Andreas Persson
+ * Copyright (C) 2006-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -91,6 +91,7 @@ public:
     BoolBox checkBoxRelease;
 
     EGStateOptions();
+    void on_show_tooltips_changed();
 };
 
 class DimRegionEdit : public Gtk::Notebook
@@ -219,6 +220,8 @@ protected:
     NumEntryTemp<uint8_t> eCrossfade_out_start;
     NumEntryTemp<uint8_t> eCrossfade_out_end;
     BoolEntry ePitchTrack;
+    ChoiceEntry<gig::sust_rel_trg_t> eSustainReleaseTrigger;
+    BoolEntry eNoNoteOffReleaseTrigger;
     ChoiceEntry<gig::dim_bypass_ctrl_t> eDimensionBypass;
     NumEntryTemp<int8_t> ePan;
     BoolEntry eSelfMask;
@@ -285,26 +288,14 @@ protected:
     void loop_length_changed();
     void loop_infinite_toggled();
     void nullOutSampleReference();
+    void on_show_tooltips_changed();
 
     int update_model;
 
-    /**
-     * Workaround for the circumstance that C++ does not allow to cast to
-     * member pointers.
-     */
-    template<typename classT, typename memberT>
-    union ClassMemberPtr {
-        memberT classT::*pmember;
-        void* pvoid;
-
-        ClassMemberPtr(size_t s) : pvoid((void*)s) {}
-        ClassMemberPtr(void* p) : pvoid(p) {}
-    };
-
     // connect a widget to a setter function in DimRegionEdit
     template<typename C, typename T>
     void connect(C& widget,
-                 void (DimRegionEdit::*setter)(gig::DimensionRegion*, T)) {
+                 void (DimRegionEdit::*setter)(gig::DimensionRegion&, T)) {
         connect<C, T>(widget,
                       sigc::mem_fun(setter));
     }
@@ -316,6 +307,13 @@ protected:
                       sigc::bind(sigc::mem_fun(&DimRegionEdit::set_member<T>), member));
     }
 
+    // connect a widget to a member of a struct member in gig::DimensionRegion
+    template<typename C, typename T, typename S>
+    void connect(C& widget, S gig::DimensionRegion::* member, T S::* member2) {
+        connect<C, T>(widget,
+                      sigc::bind(sigc::mem_fun(&DimRegionEdit::set_sub_member<T, S>), member, member2));
+    }
+
     // connect a widget to a setter function in gig::DimensionRegion
     template<typename C, typename T>
     void connect(C& widget,
@@ -327,7 +325,7 @@ protected:
     // helper function for the connect functions above
     template<typename C, typename T>
     void connect(C& widget,
-                 sigc::slot<void, DimRegionEdit*, gig::DimensionRegion*, T> setter) {
+                 sigc::slot<void, DimRegionEdit&, gig::DimensionRegion&, T> setter) {
         widget.signal_value_changed().connect(
             sigc::compose(sigc::bind(sigc::mem_fun(*this, &DimRegionEdit::set_many<T>), setter),
                           sigc::mem_fun(widget, &C::get_value)));
@@ -337,39 +335,46 @@ protected:
     // each of them
     template<typename T>
     void set_many(T value,
-                  sigc::slot<void, DimRegionEdit*, gig::DimensionRegion*, T> setter) {
+                  sigc::slot<void, DimRegionEdit&, gig::DimensionRegion&, T> setter) {
         if (update_model == 0) {
             for (std::set<gig::DimensionRegion*>::iterator i = dimregs.begin() ;
                  i != dimregs.end() ; ++i)
             {
                 DimRegionChangeGuard(this, *i);
-                setter(this, *i, value);
+                setter(*this, **i, value);
             }
         }
     }
 
     // set a value of a member variable in the given dimregion
     template<typename T>
-    void set_member(gig::DimensionRegion* d, T value,
+    void set_member(gig::DimensionRegion& d, T value,
                     T gig::DimensionRegion::* member) {
-        d->*member = value;
+        d.*member = value;
+    }
+
+    // set a value of a member of a struct member variable in the given dimregion
+    template<typename T, typename S>
+    void set_sub_member(gig::DimensionRegion& d, T value,
+                        S gig::DimensionRegion::* member, T S::* member2) {
+        d.*member.*member2 = value;
     }
 
     // setters for specific dimregion parameters
 
-    void set_UnityNote(gig::DimensionRegion* d, uint8_t value);
-    void set_FineTune(gig::DimensionRegion* d, int16_t value);
-    void set_Crossfade_in_start(gig::DimensionRegion* d, uint8_t value);
-    void set_Crossfade_in_end(gig::DimensionRegion* d, uint8_t value);
-    void set_Crossfade_out_start(gig::DimensionRegion* d, uint8_t value);
-    void set_Crossfade_out_end(gig::DimensionRegion* d, uint8_t value);
-    void set_Gain(gig::DimensionRegion* d, int32_t value);
-    void set_LoopEnabled(gig::DimensionRegion* d, bool value);
-    void set_LoopType(gig::DimensionRegion* d, uint32_t value);
-    void set_LoopStart(gig::DimensionRegion* d, uint32_t value);
-    void set_LoopLength(gig::DimensionRegion* d, uint32_t value);
-    void set_LoopInfinite(gig::DimensionRegion* d, bool value);
-    void set_LoopPlayCount(gig::DimensionRegion* d, uint32_t value);
+    void set_UnityNote(gig::DimensionRegion& d, uint8_t value);
+    void set_FineTune(gig::DimensionRegion& d, int16_t value);
+    void set_Crossfade_in_start(gig::DimensionRegion& d, uint8_t value);
+    void set_Crossfade_in_end(gig::DimensionRegion& d, uint8_t value);
+    void set_Crossfade_out_start(gig::DimensionRegion& d, uint8_t value);
+    void set_Crossfade_out_end(gig::DimensionRegion& d, uint8_t value);
+    void set_Gain(gig::DimensionRegion& d, int32_t value);
+    void set_LoopEnabled(gig::DimensionRegion& d, bool value);
+    void set_LoopType(gig::DimensionRegion& d, uint32_t value);
+    void set_LoopStart(gig::DimensionRegion& d, uint32_t value);
+    void set_LoopLength(gig::DimensionRegion& d, uint32_t value);
+    void set_LoopInfinite(gig::DimensionRegion& d, bool value);
+    void set_LoopPlayCount(gig::DimensionRegion& d, uint32_t value);
 
     void onButtonSelectSamplePressed();
 };
