@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2017 Andreas Persson
+ * Copyright (C) 2007-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,15 +18,17 @@
  */
 
 #include "compat.h"
-// threads.h must be included first to be able to build with
-// G_DISABLE_DEPRECATED
-#if (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION == 31 && GLIBMM_MICRO_VERSION >= 2) || \
-    (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION > 31) || GLIBMM_MAJOR_VERSION > 2
-#include <glibmm/threads.h>
-#endif
-
 #include "gigedit.h"
 
+#ifdef GLIB_THREADS
+#ifndef OLD_THREADS
+#include <glibmm/threads.h>
+#endif
+#else
+#include <thread>
+#include <mutex>
+#include <condition_variable>
+#endif
 #if GTKMM_MAJOR_VERSION < 3
 #include <gdkmm/region.h>
 #endif
@@ -35,7 +37,7 @@
 #include <glibmm/miscutils.h>
 #include <gtkmm/main.h>
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__APPLE__)
 #include <gtkmm/icontheme.h>
 #endif
 
@@ -85,25 +87,43 @@ private:
     class Cond {
     private:
         bool pred;
+#ifdef GLIB_THREADS
         Glib::Threads::Mutex mutex;
         Glib::Threads::Cond cond;
+#else
+        std::mutex mutex;
+        std::condition_variable cond;
+#endif        
     public:
         Cond() : pred(false) { }
         void signal() {
+#ifdef GLIB_THREADS
             Glib::Threads::Mutex::Lock lock(mutex);
             pred = true;
             cond.signal();
+#else
+            std::lock_guard<std::mutex> lock(mutex);
+            pred = true;
+            cond.notify_one();
+#endif
         }
         void wait() {
+#ifdef GLIB_THREADS
             Glib::Threads::Mutex::Lock lock(mutex);
             while (!pred) cond.wait(mutex);
+#else
+            std::unique_lock<std::mutex> lock(mutex);
+            while (!pred) cond.wait(lock);
+#endif      
         }
     };
 
 #ifdef OLD_THREADS
     static Glib::StaticMutex mutex;
-#else
+#elif defined(GLIB_THREADS)
     static Glib::Threads::Mutex mutex;
+#else
+    static std::mutex mutex;
 #endif
     static Glib::Dispatcher* dispatcher;
     static GigEditState* current;
@@ -132,6 +152,7 @@ bool gigedit_installdir_is_parent = false;
 
 #ifdef __APPLE__
 std::string gigedit_localedir;
+std::string gigedit_datadir;
 #endif
 
 void init_app() {
@@ -152,25 +173,42 @@ void init_app() {
             std::string libdir = Glib::path_get_dirname(info.dli_fname);
 #endif
 
+            // newer pango versions don't use modules
+#if PANGO_VERSION_MAJOR == 1 && PANGO_VERSION_MINOR < 38
             if (Glib::getenv("PANGO_SYSCONFDIR") == "" &&
                 Glib::file_test(Glib::build_filename(libdir,
                                                      "pango/pango.modules"),
                                 Glib::FILE_TEST_EXISTS)) {
                 Glib::setenv("PANGO_SYSCONFDIR", libdir, true);
             }
+#endif
             if (Glib::getenv("GDK_PIXBUF_MODULE_FILE") == "") {
                 std::string module_file =
                     Glib::build_filename(libdir,
-                                         "gtk-2.0/gdk-pixbuf.loaders");
+                                         "gdk-pixbuf-2.0/2.10.0/loaders.cache");
                 if (Glib::file_test(module_file, Glib::FILE_TEST_EXISTS)) {
                     Glib::setenv("GDK_PIXBUF_MODULE_FILE", module_file, true);
                 }
             }
+            std::string datadir = Glib::build_filename(
+                Glib::path_get_dirname(libdir), "share");
+
+            if (Glib::file_test(datadir, Glib::FILE_TEST_EXISTS)) {
+                gigedit_datadir = datadir;
+
+                std::string localedir = Glib::build_filename(datadir, "locale");
+                if (Glib::file_test(localedir, Glib::FILE_TEST_EXISTS)) {
+                    gigedit_localedir = localedir;
+                }
+                std::string schemadir =
+                    Glib::build_filename(gigedit_datadir, "glib-2.0/schemas");
+                if (Glib::file_test(schemadir, Glib::FILE_TEST_EXISTS)) {
+                    Glib::setenv("GSETTINGS_SCHEMA_DIR", schemadir);
+                }
+            }
   //FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
   //#if HAVE_GETTEXT
-            std::string localedir = Glib::build_filename(libdir, "locale");
-            if (Glib::file_test(localedir, Glib::FILE_TEST_EXISTS)) {
-                gigedit_localedir = localedir;
+            if (!gigedit_localedir.empty()) {
                 bindtextdomain(GETTEXT_PACKAGE, gigedit_localedir.c_str());
             } else {
                 bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -237,14 +275,55 @@ void init_app() {
     }
 }
 
+
+#if GTKMM_MAJOR_VERSION >= 3
+
+/**
+ * This is required since GTK 3, because those GTK super heros came up with
+ * the clever idea to simply disable things like icons and keyboard shortcuts
+ * for menus and for buttons by default for all users, all devices and all
+ * apps. Yey! Seriously, I have no idea what came on their mind to find that
+ * was a good idea!
+ */
+void enforceGtk3Settings() {
+
+    // got no behavior change on those 2 settings, so ignoring them for now,
+    // actually I though I could use them to show the mnemonics in the GTK 3
+    // menus again, but it seems that was entirely removed from around GTK 3.10.
+    //g_object_set(gtk_settings_get_default(), "gtk-auto-mnemonics", false, NULL);
+    //g_object_set(gtk_settings_get_default(), "gtk-can-change-accels", true, NULL);
+
+    // bring back keyboard accelerators with GTK 3
+    g_object_set(gtk_settings_get_default(), "gtk-enable-accels", true, NULL);
+    g_object_set(gtk_settings_get_default(), "gtk-enable-mnemonics", true, NULL);
+
+    // bring back icons with GTK 3
+    g_object_set(gtk_settings_get_default(), "gtk-menu-images", true, NULL);
+    g_object_set(gtk_settings_get_default(), "gtk-button-images", true, NULL);
+
+    // who knows ... one day those GTK "masterminds" decide to disable tooltips by default as well
+    g_object_set(gtk_settings_get_default(), "gtk-enable-tooltips", true, NULL);
+}
+
+#endif // GTKM 3
+
+
 void init_app_after_gtk_init() {
+#if GTKMM_MAJOR_VERSION >= 3
+    enforceGtk3Settings();
+#endif
+    
 //FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
 #if (/*HAVE_GETTEXT &&*/ defined(__APPLE__))
     // Gtk::Main binds the gtk locale to a possible non-existent
     // directory. If we have bundled gtk locale files, we rebind here,
     // after the Gtk::Main constructor.
     if (!gigedit_localedir.empty()) {
+#if GTKMM_MAJOR_VERSION < 3
         bindtextdomain("gtk20", gigedit_localedir.c_str());
+#else
+        bindtextdomain("gtk30", gigedit_localedir.c_str());
+#endif
     }
 #endif
 
@@ -254,6 +333,12 @@ void init_app_after_gtk_init() {
         Gtk::IconTheme::get_default()->append_search_path(icon_dir);
     }
 #endif
+#ifdef __APPLE__
+    if (!gigedit_datadir.empty()) {
+        std::string icon_dir = Glib::build_filename(gigedit_datadir, "icons");
+        Gtk::IconTheme::get_default()->append_search_path(icon_dir);
+    }
+#endif    
 }
 
 void connect_signals(GigEdit* gigedit, MainWindow* mainwindow) {
@@ -412,8 +497,10 @@ sigc::signal<void, gig::Instrument*>& GigEdit::signal_switch_sampler_instrument(
 
 #ifdef OLD_THREADS
 Glib::StaticMutex GigEditState::mutex = GLIBMM_STATIC_MUTEX_INIT;
-#else
+#elif defined(GLIB_THREADS)
 Glib::Threads::Mutex GigEditState::mutex;
+#else
+std::mutex GigEditState::mutex;
 #endif
 Glib::Dispatcher* GigEditState::dispatcher = 0;
 GigEditState* GigEditState::current = 0;
@@ -430,7 +517,7 @@ void GigEditState::open_window() {
     connect_signals(parent, window);
     if (instrument) window->load_instrument(instrument);
 
-    window->signal_hide().connect(sigc::mem_fun(this,
+    window->signal_hide().connect(sigc::mem_fun(*this,
                                                 &GigEditState::close_window));
     window->present();
 }
@@ -536,10 +623,12 @@ void GigEditState::run(gig::Instrument* pInstrument) {
             sigc::bind(sigc::ptr_fun(&GigEditState::main_loop_run),
                        &initialized),
             false);
-  #else
+  #elif defined(GLIB_THREADS)
         Glib::Threads::Thread::create(
             sigc::bind(sigc::ptr_fun(&GigEditState::main_loop_run),
                        &initialized));
+  #else
+        new std::thread([this]() { main_loop_run(&initialized); });
   #endif
 #endif
         printf("Waiting for GUI being initialized (on main thread) ....\n"); fflush(stdout);
