@@ -1,5 +1,5 @@
 /*                                                         -*- c++ -*-
- * Copyright (C) 2006 - 2017 Andreas Persson
+ * Copyright (C) 2006 - 2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -59,9 +59,14 @@
 #include "dimregionchooser.h"
 #include "dimregionedit.h"
 #include "midirules.h"
+#ifdef GLIB_THREADS
 #ifndef OLD_THREADS
 #include <glibmm/threads.h>
 #endif
+#else
+#include <thread>
+#include <mutex>
+#endif
 #include "ManagedWindow.h"
 
 class MainWindow;
@@ -70,7 +75,6 @@ class PropDialog : public ManagedWindow,
                    public PropEditor<DLS::Info> {
 public:
     PropDialog();
-    void set_info(DLS::Info* info);
     void set_file(gig::File* file);
 
     // implementation for abstract methods of interface class "ManagedWindow"
@@ -104,7 +108,7 @@ protected:
 
     gig::File* m_file;
 
-    void onFileFormatChanged();
+    void set_FileFormat(int value);
 };
 
 class InstrumentProps : public ManagedWindow,
@@ -157,50 +161,52 @@ protected:
     Gtk::ProgressBar progressBar;
 };
 
-class Loader : public sigc::trackable {
+class LoaderSaverBase {
 public:
-    Loader(const char* filename);
     void launch();
     Glib::Dispatcher& signal_progress();
     Glib::Dispatcher& signal_finished(); ///< Finished successfully, without error.
     Glib::Dispatcher& signal_error();
     void progress_callback(float fraction);
     float get_progress();
+    void join();
     const Glib::ustring filename;
     Glib::ustring error_message;
     gig::File* gig;
 
+protected:
+    LoaderSaverBase(const Glib::ustring filename, gig::File* gig);
+
 private:
+#ifdef GLIB_THREADS
     Glib::Threads::Thread* thread;
+    Glib::Threads::Mutex progressMutex;
+#else
+    std::thread thread;
+    std::mutex progressMutex;
+#endif
     void thread_function();
+    virtual void thread_function_sub(gig::progress_t& progress) = 0;
     Glib::Dispatcher finished_dispatcher;
     Glib::Dispatcher progress_dispatcher;
     Glib::Dispatcher error_dispatcher;
-    Glib::Threads::Mutex progressMutex;
     float progress;
 };
 
-class Saver : public sigc::trackable {
+class Loader : public LoaderSaverBase {
 public:
-    Saver(gig::File* file, Glib::ustring filename = ""); ///< one argument means "save", two arguments means "save as"
-    void launch();
-    Glib::Dispatcher& signal_progress();
-    Glib::Dispatcher& signal_finished(); ///< Finished successfully, without error.
-    Glib::Dispatcher& signal_error();
-    void progress_callback(float fraction);
-    float get_progress();
-    gig::File* gig;
-    const Glib::ustring filename;
-    Glib::ustring error_message;
+    Loader(const char* filename);
 
 private:
-    Glib::Threads::Thread* thread;
-    void thread_function();
-    Glib::Dispatcher finished_dispatcher;
-    Glib::Dispatcher progress_dispatcher;
-    Glib::Dispatcher error_dispatcher;
-    Glib::Threads::Mutex progressMutex;
-    float progress;
+    void thread_function_sub(gig::progress_t& progress);
+};
+
+class Saver : public LoaderSaverBase {
+public:
+    Saver(gig::File* file, Glib::ustring filename = ""); ///< one argument means "save", two arguments means "save as"
+
+private:
+    void thread_function_sub(gig::progress_t& progress);
 };
 
 class MainWindow : public ManagedWindow {
@@ -273,6 +279,7 @@ protected:
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleRestoreWinDim;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleSaveWithTempFile;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleWarnOnExtensions;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleShowTooltips;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleSyncSamplerSelection;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleMoveRootNoteWithRegion;
 #endif
@@ -352,12 +359,14 @@ protected:
           add(m_col_name);
           add(m_col_instr);
           add(m_col_scripts);
+          add(m_col_tooltip);
         }
 
         Gtk::TreeModelColumn<int> m_col_nr;
         Gtk::TreeModelColumn<Glib::ustring> m_col_name;
         Gtk::TreeModelColumn<gig::Instrument*> m_col_instr;
         Gtk::TreeModelColumn<Glib::ustring> m_col_scripts;
+        Gtk::TreeModelColumn<Glib::ustring> m_col_tooltip;
     } m_Columns;
 
     VBox m_VBox;
@@ -484,6 +493,8 @@ protected:
     void on_save_with_temporary_file();
     void on_action_refresh_all();
     void on_action_warn_user_on_extensions();
+    void on_action_show_tooltips();
+    void on_show_tooltips_changed();
     void on_action_sync_sampler_instrument_selection();
     void on_action_move_root_note_with_region_moved();
     void on_action_help_about();
@@ -561,6 +572,7 @@ protected:
                                                int position = -1);
     void remove_instrument_from_menu(int index);
 #endif
+    bool onQueryTreeViewTooltip(int x, int y, bool keyboardTip, const Glib::RefPtr<Gtk::Tooltip>& tooltip);
 
     ProgressDialog* progress_dialog;
     Loader* loader;
@@ -632,6 +644,7 @@ protected:
     void __refreshEntireGUI();
     void updateScriptListOfMenu();
     void assignScript(gig::Script* pScript);
+    void dropAllScriptSlots();
 
     bool close_confirmation_dialog();
     bool leaving_shared_mode_dialog();
@@ -645,7 +658,7 @@ protected:
     bool on_delete_event(GdkEventAny* event);
 
     bool first_call_to_drag_data_get;
-    
+
     bool is_copy_samples_unity_note_enabled() const;
     bool is_copy_samples_fine_tune_enabled() const;
     bool is_copy_samples_loop_enabled() const;
