@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2017 Andreas Persson
+ * Copyright (C) 2006-2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -269,7 +269,7 @@ RegionChooser::RegionChooser() :
 
 #endif // USE_GTKMM_BUILDER
 
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
 # warning GTKMM4 event registration code missing for regionchooser!
     //add_events(Gdk::EventMask::BUTTON_PRESS_MASK);
 #else
@@ -295,12 +295,24 @@ RegionChooser::RegionChooser() :
         sigc::mem_fun(*this, &RegionChooser::on_note_off_event)
     );
     set_tooltip_text(_("Right click here for adding new region. Use mouse pointer for moving (dragging) or resizing existing regions (by pointing at region's boundary). Right click on an existing region for more actions."));
+
+    Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
+        sigc::mem_fun(*this, &RegionChooser::on_show_tooltips_changed)
+    );
+
+    on_show_tooltips_changed();
 }
 
 RegionChooser::~RegionChooser()
 {
 }
 
+void RegionChooser::on_show_tooltips_changed() {
+    const bool b = Settings::singleton()->showTooltips;
+
+    set_has_tooltip(b);
+}
+
 void RegionChooser::setModifyAllRegions(bool b) {
     modifyallregions = b;
     // redraw required parts
@@ -354,7 +366,7 @@ bool RegionChooser::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
     const Gdk::Color bg = get_style()->get_bg(Gtk::STATE_NORMAL);
 #else
-#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 24)
     GdkRGBA gdkBgRGBA;
     gtk_style_context_get_background_color(get_style_context()->gobj(), &gdkBgRGBA);
     const Gdk::RGBA bg = Glib::wrap(&gdkBgRGBA, true);
@@ -662,7 +674,7 @@ bool RegionChooser::on_button_release_event(GdkEventButton* event)
 # if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
         Glib::wrap(event->device, true)->ungrab(event->time);
 # else
-        gdk_device_ungrab(Glib::wrap(event->device, true)->gobj(), event->time);
+        Glib::wrap(event->device, true)->get_seat()->ungrab();
 # endif
 #endif
         resize.active = false;
@@ -678,7 +690,7 @@ bool RegionChooser::on_button_release_event(GdkEventButton* event)
 # if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
         Glib::wrap(event->device, true)->ungrab(event->time);
 # else
-        gdk_device_ungrab(Glib::wrap(event->device, true)->gobj(), event->time);
+        Glib::wrap(event->device, true)->get_seat()->ungrab();
 # endif
 #endif
         move.active = false;
@@ -802,18 +814,15 @@ bool RegionChooser::on_button_press_event(GdkEventButton* event)
                                                   Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW),
                                                   event->time);
 # else
-            gdk_device_grab(
-                Glib::wrap(event->device, true)->gobj(),
-                get_window()->gobj(),
-                GDK_OWNERSHIP_NONE,
+            Glib::wrap(event->device, true)->get_seat()->grab(
+                get_window(),
+                Gdk::SeatCapabilities::SEAT_CAPABILITY_ALL_POINTING,
                 false,
-                GdkEventMask(GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
-                             GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK),
                 Gdk::Cursor::create(
                     Glib::wrap(event->device, true)->get_seat()->get_display(),
                     Gdk::SB_H_DOUBLE_ARROW
-                )->gobj(),
-                event->time
+                ),
+                reinterpret_cast<GdkEvent*>(event)
             );
 # endif
 #endif
@@ -844,18 +853,15 @@ bool RegionChooser::on_button_press_event(GdkEventButton* event)
                                                   Gdk::Cursor::create(Gdk::FLEUR),
                                                   event->time);
 # else
-            gdk_device_grab(
-                Glib::wrap(event->device, true)->gobj(),
-                get_window()->gobj(),
-                GDK_OWNERSHIP_NONE,
+            Glib::wrap(event->device, true)->get_seat()->grab(
+                get_window(),
+                Gdk::SeatCapabilities::SEAT_CAPABILITY_ALL_POINTING,
                 false,
-                GdkEventMask(GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
-                             GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK),
                 Gdk::Cursor::create(
                     Glib::wrap(event->device, true)->get_seat()->get_display(),
                     Gdk::FLEUR
-                )->gobj(),
-                event->time
+                ),
+                reinterpret_cast<GdkEvent*>(event)
             );
 # endif
 #endif
