@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2017 Andreas Persson
+ * Copyright (C) 2007 - 2019 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -177,7 +177,7 @@ int LinuxSamplerPlugin::Main(void* pInstrument, String sTypeName, String sTypeVe
     const Glib::RefPtr<Glib::TimeoutSource> timeout_source =
         Glib::TimeoutSource::create(100); // poll every 100ms
     timeout_source->connect(
-        sigc::mem_fun(this, &LinuxSamplerPlugin::__onPollPeriod)
+        sigc::mem_fun(*this, &LinuxSamplerPlugin::__onPollPeriod)
     );
     timeout_source->attach(Glib::MainContext::get_default());
 
